@@ -1087,6 +1087,7 @@ void samplv1_impl::process_midi ( uint8_t *data, uint32_t size )
 				m_dca1.env.note_off(&pv->dca1_env);
 				m_dcf1.env.note_off(&pv->dcf1_env);
 				m_lfo1.env.note_off(&pv->lfo1_env);
+				pv->gen1.setLoop(false);
 			}
 		}
 	}
@@ -1234,8 +1235,8 @@ void samplv1_impl::process ( float **ins, float **outs, uint32_t nframes )
 		gen1_sample.reset(note_freq(m_gen1.sample0));
 	}
 
-	if (bool(int(*m_gen1.loop)) != gen1_sample.loop())
-		gen1_sample.setLoop(int(*m_gen1.loop));
+	if (bool(int(*m_gen1.loop)) != gen1_sample.isLoop())
+		gen1_sample.setLoop(bool(*m_gen1.loop));
 
 	if (int(*m_lfo1.shape) != int(lfo1_wave.shape()) || *m_lfo1.width != lfo1_wave.width())
 		lfo1_wave.reset(samplv1_wave::Shape(*m_lfo1.shape), *m_lfo1.width);
@@ -1471,6 +1472,22 @@ samplv1_sample *samplv1::sample (void) const
 }
 
 
+void samplv1::setLoopRange ( uint32_t iLoopStart, uint32_t iLoopEnd )
+{
+	m_pImpl->gen1_sample.setLoopRange(iLoopStart, iLoopEnd);
+}
+
+uint32_t samplv1::loopStart (void) const
+{
+	return m_pImpl->gen1_sample.loopStart();
+}
+
+uint32_t samplv1::loopEnd (void) const
+{
+	return m_pImpl->gen1_sample.loopEnd();
+}
+
+
 void samplv1::setParamPort ( ParamIndex index, float *pfParam )
 {
 	m_pImpl->setParamPort(index, pfParam);
