@@ -54,6 +54,10 @@ public:
 
 	samplv1_sample *sample() const;
 
+	void setLoopRange(uint32_t iLoopStart, uint32_t iLoopEnd);
+	uint32_t loopStart() const;
+	uint32_t loopEnd() const;
+
 	enum ParamIndex	 {
 
 		GEN1_SAMPLE = 0,
