@@ -21,6 +21,8 @@
 
 #include "samplv1widget.h"
 
+#include "samplv1_sample.h"
+
 #include <QDomDocument>
 #include <QTextStream>
 #include <QFileInfo>
@@ -123,6 +125,24 @@ samplv1widget::samplv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 	// Start clean.
 	m_iUpdate = 0;
 
+	// Replicate the stacked/pages
+	for (int iTab = 0; iTab < m_ui.StackedWidget->count(); ++iTab)
+		m_ui.TabBar->addTab(m_ui.StackedWidget->widget(iTab)->windowTitle());
+
+	// Loop range font.
+	const QFont& font = m_ui.Gen1LoopKnob->font();
+	m_ui.Gen1LoopRangeFrame->setFont(font);
+
+	const QFontMetrics fm(font);
+	m_ui.Gen1LoopStartSpinBox->setMaximumHeight(fm.height() + 6);
+	m_ui.Gen1LoopEndSpinBox->setMaximumHeight(fm.height() + 6);
+
+	m_ui.Gen1LoopStartSpinBox->setAccelerated(true);
+	m_ui.Gen1LoopEndSpinBox->setAccelerated(true);
+
+	m_ui.Gen1LoopStartSpinBox->setMinimum(0);
+	m_ui.Gen1LoopEndSpinBox->setMinimum(0);
+
 	// Note names.
 	QStringList notes;
 	for (int note = 0; note < 128; ++note)
@@ -446,6 +466,16 @@ samplv1widget::samplv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 		SIGNAL(customContextMenuRequested(const QPoint&)),
 		SLOT(contextMenuRequest(const QPoint&)));
 
+	QObject::connect(m_ui.Gen1Sample,
+		SIGNAL(loopRangeChanged()),
+		SLOT(loopRangeChanged()));
+
+	QObject::connect(m_ui.Gen1LoopStartSpinBox,
+		SIGNAL(valueChanged(int)),
+		SLOT(loopStartChanged()));
+	QObject::connect(m_ui.Gen1LoopEndSpinBox,
+		SIGNAL(valueChanged(int)),
+		SLOT(loopEndChanged()));
 
 	// Swap params A/B
 	QObject::connect(m_ui.SwapParamsAButton,
@@ -455,6 +485,9 @@ samplv1widget::samplv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 		SIGNAL(toggled(bool)),
 		SLOT(swapParams(bool)));
 
+	// Direct stacked-page signal/slot
+	QObject::connect(m_ui.TabBar, SIGNAL(currentChanged(int)),
+		m_ui.StackedWidget, SLOT(setCurrentIndex(int)));
 
 	// Menu actions
 	QObject::connect(m_ui.helpAboutAction,
@@ -464,9 +497,11 @@ samplv1widget::samplv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 		SIGNAL(triggered(bool)),
 		SLOT(helpAboutQt()));
 
-
 	// Epilog.
-	QWidget::adjustSize();
+	// QWidget::adjustSize();
+
+	m_ui.StatusBar->showMessage(tr("Ready"), 5000);
+	m_ui.StatusBar->setModified(false);
 }
 
 
@@ -496,6 +531,8 @@ void samplv1widget::setParamValue ( samplv1::ParamIndex index, float fValue )
 	if (pKnob)
 		pKnob->setValue(fValue);
 
+	updateParamEx(index, fValue);
+
 	--m_iUpdate;
 }
 
@@ -513,13 +550,37 @@ void samplv1widget::paramChanged ( float fValue )
 		return;
 
 	samplv1widget_knob *pKnob = qobject_cast<samplv1widget_knob *> (sender());
-	if (pKnob)
-		updateParam(m_knobParams.value(pKnob), fValue);
+	if (pKnob) {
+		samplv1::ParamIndex index = m_knobParams.value(pKnob);
+		updateParam(index, fValue);
+		updateParamEx(index, fValue);
+		m_ui.StatusBar->showMessage(QString("%1 / %2: %3")
+			.arg(m_ui.StackedWidget->currentWidget()->windowTitle())
+			.arg(pKnob->toolTip())
+			.arg(pKnob->valueText()), 5000);
+		m_ui.StatusBar->setModified(true);
+	}
 
 	m_ui.Preset->dirtyPreset();
 }
 
 
+// Update local tied widgets.
+void samplv1widget::updateParamEx ( samplv1::ParamIndex index, float fValue )
+{
+	switch (index) {
+	case samplv1::GEN1_LOOP: {
+		const bool bLoop = bool(fValue > 0.0f);
+		m_ui.Gen1Sample->setLoop(bLoop);
+		m_ui.Gen1LoopRangeFrame->setEnabled(bLoop);
+		// Fall thru...
+	}
+	default:
+		break;
+	}
+}
+
+
 // Reset all param knobs to default values.
 void samplv1widget::resetParams (void)
 {
@@ -535,6 +596,9 @@ void samplv1widget::resetParams (void)
 		updateParam(index, fValue);
 		m_params_ab[index] = fValue;
 	}
+
+	m_ui.StatusBar->showMessage(tr("Reset preset"), 5000);
+	m_ui.StatusBar->setModified(false);
 }
 
 
@@ -562,6 +626,10 @@ void samplv1widget::swapParams ( bool bOn )
 	}
 
 	m_ui.Preset->dirtyPreset();
+
+	const bool bSwapA = m_ui.SwapParamsAButton->isChecked();
+	m_ui.StatusBar->showMessage(tr("Swap %1").arg(bSwapA ? 'A' : 'B'), 5000);
+	m_ui.StatusBar->setModified(true);
 }
 
 
@@ -608,6 +676,13 @@ void samplv1widget::initPreset (void)
 
 
 // Preset clear.
+void samplv1widget::clearPreset (void)
+{
+	m_ui.Preset->clearPreset();
+}
+
+
+// Preset renewal.
 void samplv1widget::newPreset (void)
 {
 #ifdef CONFIG_DEBUG
@@ -619,6 +694,9 @@ void samplv1widget::newPreset (void)
 	resetParamKnobs();
 	resetParamValues();
 
+	m_ui.StatusBar->showMessage(tr("New preset"), 5000);
+	m_ui.StatusBar->setModified(false);
+
 	m_ui.Gen1Sample->openSample();
 }
 
@@ -661,17 +739,7 @@ void samplv1widget::loadPreset ( const QString& sFilename )
 				if (eChild.isNull())
 					continue;
 				if (eChild.tagName() == "samples") {
-					for (QDomNode nSample = eChild.firstChild();
-							!nSample.isNull();
-								nSample = nSample.nextSibling()) {
-						QDomElement eSample = nSample.toElement();
-						if (eSample.isNull())
-							continue;
-						if (eSample.tagName() == "sample") {
-						//	int index = eSample.attribute("index").toInt();
-							loadSampleFile(eSample.text());
-						}
-					}
+					loadSamples(eChild);
 				}
 				else
 				if (eChild.tagName() == "params") {
@@ -700,7 +768,11 @@ void samplv1widget::loadPreset ( const QString& sFilename )
 
 	file.close();
 
-	m_ui.Preset->setPreset(fi.completeBaseName());
+	const QString& sPreset = fi.completeBaseName();
+	m_ui.Preset->setPreset(sPreset);
+
+	m_ui.StatusBar->showMessage(tr("Load preset: %1").arg(sPreset), 5000);
+	m_ui.StatusBar->setModified(false);
 
 	QDir::setCurrent(currentDir.absolutePath());
 }
@@ -711,17 +783,15 @@ void samplv1widget::savePreset ( const QString& sFilename )
 #ifdef CONFIG_DEBUG
 	qDebug("samplv1widget::savePreset(\"%s\")", sFilename.toUtf8().constData());
 #endif
+	const QString& sPreset = QFileInfo(sFilename).completeBaseName();
+
 	QDomDocument doc(SAMPLV1_TITLE);
 	QDomElement ePreset = doc.createElement("preset");
-	ePreset.setAttribute("name", QFileInfo(sFilename).completeBaseName());
+	ePreset.setAttribute("name", sPreset);
 	ePreset.setAttribute("version", SAMPLV1_VERSION);
 
 	QDomElement eSamples = doc.createElement("samples");
-	QDomElement eSample = doc.createElement("sample");
-	eSample.setAttribute("index", 0);
-	eSample.setAttribute("name", "GEN1_SAMPLE");
-	eSample.appendChild(doc.createTextNode(sampleFile()));
-	eSamples.appendChild(eSample);
+	saveSamples(doc, eSamples);
 	ePreset.appendChild(eSamples);
 
 	QDomElement eParams = doc.createElement("params");
@@ -741,6 +811,96 @@ void samplv1widget::savePreset ( const QString& sFilename )
 		QTextStream(&file) << doc.toString();
 		file.close();
 	}
+
+	m_ui.StatusBar->showMessage(tr("Save preset: %1").arg(sPreset), 5000);
+	m_ui.StatusBar->setModified(false);
+}
+
+
+// Sample serialization methods.
+void samplv1widget::loadSamples ( const QDomElement& eSamples )
+{
+	samplv1 *pSampl = instance();
+	if (pSampl == NULL)
+		return;
+
+	for (QDomNode nSample = eSamples.firstChild();
+			!nSample.isNull();
+				nSample = nSample.nextSibling()) {
+		QDomElement eSample = nSample.toElement();
+		if (eSample.isNull())
+			continue;
+		if (eSample.tagName() == "sample") {
+		//	int index = eSample.attribute("index").toInt();
+			QString sFilename;
+			uint32_t iLoopStart = 0;
+			uint32_t iLoopEnd = 0;
+			for (QDomNode nChild = eSample.firstChild();
+					!nChild.isNull();
+						nChild = nChild.nextSibling()) {
+				QDomElement eChild = nChild.toElement();
+				if (eChild.isNull())
+					continue;
+				if (eChild.tagName() == "filename") {
+				//	int index = eSample.attribute("index").toInt();
+					sFilename = eChild.text();
+				}
+				else
+				if (eChild.tagName() == "loop-start") {
+					iLoopStart = eChild.text().toULong();
+				}
+				else
+				if (eChild.tagName() == "loop-end") {
+					iLoopEnd = eChild.text().toULong();
+				}
+			}
+			// Legacy loader...
+			if (sFilename.isEmpty())
+				sFilename = eSample.text();
+			// Done it.
+			loadSampleFile(sFilename);
+			setSampleLoop(iLoopStart, iLoopEnd);
+		}
+	}
+
+	pSampl->reset();
+}
+
+
+void samplv1widget::saveSamples (
+	QDomDocument& doc, QDomElement& eSamples )
+{
+	samplv1 *pSampl = instance();
+	if (pSampl == NULL)
+		return;
+
+	const char *pszSampleFile = pSampl->sampleFile();
+	if (pszSampleFile == NULL)
+		return;
+
+	QDomElement eSample = doc.createElement("sample");
+	eSample.setAttribute("index", 0);
+	eSample.setAttribute("name", "GEN1_SAMPLE");
+
+	QDomElement eFilename = doc.createElement("filename");
+	eFilename.appendChild(doc.createTextNode(
+		QString::fromUtf8(pszSampleFile)));
+	eSample.appendChild(eFilename);
+
+	uint32_t iLoopStart = pSampl->loopStart();
+	uint32_t iLoopEnd   = pSampl->loopEnd();
+	if (iLoopStart < iLoopEnd) {
+		QDomElement eLoopStart = doc.createElement("loop-start");
+		eLoopStart.appendChild(doc.createTextNode(
+			QString::number(iLoopStart)));
+		eSample.appendChild(eLoopStart);
+		QDomElement eLoopEnd = doc.createElement("loop-end");
+		eLoopEnd.appendChild(doc.createTextNode(
+			QString::number(iLoopEnd)));
+		eSample.appendChild(eLoopEnd);
+	}
+
+	eSamples.appendChild(eSample);
 }
 
 
@@ -749,6 +909,9 @@ void samplv1widget::clearSample (void)
 {
 	clearSampleFile();
 
+	m_ui.StatusBar->showMessage(tr("Clear sample"), 5000);
+	m_ui.StatusBar->setModified(true);
+
 	m_ui.Preset->dirtyPreset();
 }
 
@@ -758,6 +921,9 @@ void samplv1widget::loadSample ( const QString& sFilename )
 {
 	loadSampleFile(sFilename);
 
+	m_ui.StatusBar->showMessage(tr("Load sample: %1").arg(sFilename), 5000);
+	m_ui.StatusBar->setModified(true);
+
 	m_ui.Preset->dirtyPreset();
 }
 
@@ -799,27 +965,59 @@ void samplv1widget::loadSampleFile ( const QString& sFilename )
 }
 
 
-// Sample filename retriever (crude experimental stuff III).
-QString samplv1widget::sampleFile (void) const
-{
-	samplv1 *pSampl = instance();
-	if (pSampl)
-		return QString::fromUtf8(pSampl->sampleFile());
-	else
-		return QString();
-}
-
-
 // Sample updater (crude experimental stuff II).
 void samplv1widget::updateSample ( samplv1_sample *pSample, bool bDirty )
 {
 	m_ui.Gen1Sample->setSample(pSample);
 
+	++m_iUpdate;
+	if (pSample) {
+		const bool bLoop = pSample->isLoop();
+		const uint32_t nframes = pSample->length();
+		m_ui.Gen1Sample->setLoop(bLoop);
+		m_ui.Gen1Sample->setLoopStart(0);
+		m_ui.Gen1Sample->setLoopEnd(nframes);
+		m_ui.Gen1LoopStartSpinBox->setMaximum(nframes);
+		m_ui.Gen1LoopStartSpinBox->setValue(0);
+		m_ui.Gen1LoopEndSpinBox->setMaximum(nframes);
+		m_ui.Gen1LoopEndSpinBox->setValue(nframes);
+	} else {
+		m_ui.Gen1Sample->setLoop(false);
+		m_ui.Gen1LoopStartSpinBox->setMaximum(0);
+		m_ui.Gen1LoopStartSpinBox->setValue(0);
+		m_ui.Gen1LoopEndSpinBox->setMaximum(0);
+		m_ui.Gen1LoopEndSpinBox->setValue(0);
+	}
+	--m_iUpdate;
+
 	if (pSample && bDirty)
 		m_ui.Preset->dirtyPreset();
 }
 
 
+// Sample loop points.
+void samplv1widget::setSampleLoop ( uint32_t iLoopStart, uint32_t iLoopEnd )
+{
+	if (iLoopStart < iLoopEnd) {
+		++m_iUpdate;
+		samplv1 *pSampl = instance();
+		if (pSampl) {
+			pSampl->setLoopRange(iLoopStart, iLoopEnd);
+			m_ui.Gen1Sample->setLoopStart(iLoopStart);
+			m_ui.Gen1Sample->setLoopEnd(iLoopEnd);
+			const uint32_t nframes = pSampl->sample()->length();
+			m_ui.Gen1LoopStartSpinBox->setMaximum(
+				iLoopEnd > 0 ? iLoopEnd - 1 : 0);
+			m_ui.Gen1LoopEndSpinBox->setMinimum(
+				iLoopStart < nframes ? iLoopStart + 1 : nframes);
+			m_ui.Gen1LoopStartSpinBox->setValue(iLoopStart);
+			m_ui.Gen1LoopEndSpinBox->setValue(iLoopEnd);
+		}
+		--m_iUpdate;
+	}
+}
+
+
 // MIDI note/octave name helper (static).
 QString samplv1widget::noteName ( int note )
 {
@@ -836,6 +1034,83 @@ bool samplv1widget::queryClose (void)
 }
 
 
+// Loop range change.
+void samplv1widget::loopRangeChanged (void)
+{
+	if (m_iUpdate > 0)
+		return;
+
+	++m_iUpdate;
+	samplv1 *pSampl = instance();
+	if (pSampl) {
+		const uint32_t iLoopStart = m_ui.Gen1Sample->loopStart();
+		const uint32_t iLoopEnd = m_ui.Gen1Sample->loopEnd();
+		pSampl->setLoopRange(iLoopStart, iLoopEnd);
+		const uint32_t nframes = pSampl->sample()->length();
+		m_ui.Gen1LoopStartSpinBox->setMaximum(
+			iLoopEnd > 0 ? iLoopEnd - 1 : 0);
+		m_ui.Gen1LoopEndSpinBox->setMinimum(
+			iLoopStart < nframes ? iLoopStart + 1 : nframes);
+		m_ui.Gen1LoopStartSpinBox->setValue(iLoopStart);
+		m_ui.Gen1LoopEndSpinBox->setValue(iLoopEnd);
+		updateLoopRange(iLoopStart, iLoopEnd);
+	}
+	--m_iUpdate;
+}
+
+
+// Loop start change.
+void samplv1widget::loopStartChanged (void)
+{
+	if (m_iUpdate > 0)
+		return;
+
+	++m_iUpdate;
+	samplv1 *pSampl = instance();
+	if (pSampl) {
+		const uint32_t iLoopStart = m_ui.Gen1LoopStartSpinBox->value();
+		const uint32_t iLoopEnd = pSampl->loopEnd();
+		pSampl->setLoopRange(iLoopStart, iLoopEnd);
+		m_ui.Gen1Sample->setLoopStart(iLoopStart);
+		const uint32_t nframes = pSampl->sample()->length();
+		m_ui.Gen1LoopEndSpinBox->setMinimum(
+			iLoopStart < nframes ? iLoopStart + 1 : nframes);
+		updateLoopRange(iLoopStart, iLoopEnd);
+	}
+	--m_iUpdate;
+}
+
+
+// Loop end change.
+void samplv1widget::loopEndChanged (void)
+{
+	if (m_iUpdate > 0)
+		return;
+
+	++m_iUpdate;
+	samplv1 *pSampl = instance();
+	if (pSampl) {
+		const uint32_t iLoopStart = pSampl->loopStart();
+		const uint32_t iLoopEnd = m_ui.Gen1LoopEndSpinBox->value();
+		pSampl->setLoopRange(iLoopStart, iLoopEnd);
+		m_ui.Gen1Sample->setLoopEnd(iLoopEnd);
+		m_ui.Gen1LoopStartSpinBox->setMaximum(
+			iLoopEnd > 0 ? iLoopEnd - 1 : 0);
+		updateLoopRange(iLoopStart, iLoopEnd);
+	}
+	--m_iUpdate;
+}
+
+
+void samplv1widget::updateLoopRange ( uint32_t iLoopStart, uint32_t iLoopEnd )
+{
+	m_ui.Preset->dirtyPreset();
+	m_ui.StatusBar->showMessage(tr("Loop start: %1, end: %2")
+		.arg(iLoopStart).arg(iLoopEnd), 5000);
+	m_ui.StatusBar->setModified(true);
+}
+
+
 // Sample context menu.
 void samplv1widget::contextMenuRequest ( const QPoint& pos )
 {
