@@ -31,6 +31,11 @@
 #include <QHash>
 
 
+// forward decl.
+class QDomElement;
+class QDomDocument;
+
+
 //-------------------------------------------------------------------------
 // samplv1widget - decl.
 //
@@ -55,6 +60,11 @@ public:
 	// MIDI note/octave name helper.
 	static QString noteName(int note);
 
+	// Preset init.
+	void initPreset();
+	// Preset clear.
+	void clearPreset();
+
 	// Dirty close prompt,
 	bool queryClose();
 
@@ -66,7 +76,7 @@ public slots:
 
 protected slots:
 
-	// Preset clear.
+	// Preset renewal.
 	void newPreset();
 
 	// Param knob (widget) slots.
@@ -90,6 +100,11 @@ protected slots:
 	// Swap params A/B.
 	void swapParams(bool bOn);
 
+	// Loop range changes.
+	void loopRangeChanged();
+	void loopStartChanged();
+	void loopEndChanged();
+
 	// Menu actions.
 	void helpAbout();
 	void helpAboutQt();
@@ -99,9 +114,6 @@ protected:
 	// Synth engine accessor.
 	virtual samplv1 *instance() const = 0;
 
-	// Preset init.
-	void initPreset();
-
 	// Reset swap params A/B group.
 	void resetSwapParams();
 
@@ -112,18 +124,28 @@ protected:
 	// Param port methods.
 	virtual void updateParam(samplv1::ParamIndex index, float fValue) const = 0;
 
+	// Update local tied widgets.
+	void updateParamEx(samplv1::ParamIndex index, float fValue);
+
 	// Sample file clearance.
 	void clearSampleFile();
 
 	// Sample loader slot.
 	void loadSampleFile(const QString& sFilename);
 
-	// Sample filename retriever.
-	QString sampleFile() const;
-
 	// Sample updater.
 	void updateSample(samplv1_sample *pSample, bool bDirty = false);
 
+	// Sample loop points.
+	void setSampleLoop(uint32_t iLoopStart, uint32_t iLoopEnd);
+
+	// Update loop range change status.
+	void updateLoopRange(uint32_t iLoopStart, uint32_t iLoopEnd);
+
+	// Sample serialization methods.
+	void loadSamples(const QDomElement& eSamples);
+	void saveSamples(QDomDocument& doc, QDomElement& eSamples);
+
 private:
 
 	// Instance variables.
