@@ -62,6 +62,8 @@ public:
 	void setDefaultValue(float fDefaultValue);
 	float defaultValue() const;
 
+	virtual QString valueText() const;
+
 public slots:
 
 	virtual void setValue(float fValue);
@@ -144,6 +146,8 @@ public:
 	void insertItems(int iIndex, const QStringList& items);
 	void clear();
 
+	QString valueText() const;
+
 public slots:
 
 	// Virtual accessor.
