@@ -24,6 +24,8 @@
 
 #include <QFrame>
 
+#include <stdint.h>
+
 
 // Forward decl.
 class samplv1_sample;
@@ -47,11 +49,24 @@ public:
 	void setSample(samplv1_sample *pSample);
 	samplv1_sample *sample() const;
 
+	void setSampleName(const QString& sName);
+	const QString& sampleName() const;
+
+	void setLoop(bool bLoop);
+	bool isLoop() const;
+
+	// Loop point getters.
+	uint32_t loopStart() const;
+	uint32_t loopEnd() const;
+
 signals:
 
 	// Load new sample file.
 	void loadSampleFile(const QString&);
 
+	// Loop range changed.
+	void loopRangeChanged();
+
 public slots:
 
 	// Browse for a new sample.
@@ -60,23 +75,61 @@ public slots:
 	// Effective sample slot.
 	void loadSample(samplv1_sample *pSample);
 
+	// Loop point setters.
+	void setLoopStart(uint32_t iLoopStart);
+	void setLoopEnd(uint32_t iLoopEnd);
+
 protected:
 
 	// Widget resize handler.
 	void resizeEvent(QResizeEvent *);
 
-	// Mouse interaction.
-	void mouseDoubleClickEvent(QMouseEvent *pMouseEvent);
-
 	// Draw canvas.
 	void paintEvent(QPaintEvent *);
 
+	// Mouse interaction.
+	void mousePressEvent(QMouseEvent *pMouseEvent);
+	void mouseMoveEvent(QMouseEvent *pMouseEvent);
+	void mouseReleaseEvent(QMouseEvent *pMouseEvent);
+
+	void mouseDoubleClickEvent(QMouseEvent *pMouseEvent);
+
+	// Trap for escape key.
+	void keyPressEvent(QKeyEvent *pKeyEvent);
+
+	// Sanitizer helper;
+	int safeX(int x) const;
+
+	// Reset drag/select state.
+	void resetDragState();
+
+	// Update tool-tip.
+	void updateToolTip();
+
 private:
 
 	// Instance state.
 	samplv1_sample *m_pSample;
 	unsigned short m_iChannels;
 	QPolygon **m_ppPolyg;
+
+	QString m_sName;
+
+	// Drag state.
+	enum DragState {
+		DragNone = 0, DragStart, DragSelect,
+		DragLoopStart, DragLoopEnd,
+	} m_dragState, m_dragCursor;
+
+	QPoint m_posDrag;
+
+	int m_iDragStartX;
+	int m_iDragEndX;
+
+	// Loop state.
+	bool     m_bLoop;
+	uint32_t m_iLoopStart;
+	uint32_t m_iLoopEnd;
 };
 
 #endif	// __samplv1widget_sample_h
